'use strict';

const express = require('express');
const http = require('http');
const bearerToken = require('express-bearer-token');
const cookieParser = require('cookie-parser');
const bodyParser = require('body-parser');
const serveStatic = require('serve-static');
const initRoutes = require('../routes');
const sentry = require('./sentry');
const pino = require('./pino');
// const AccessToken = require('../old_models/access-token');
const uuidv4 = require('uuid/v4');

const devMode = process.env.APP_ENV === 'development';

const httpLogger = require('pino-http')({
    logger: pino,
    genReqId: () => uuidv4(),
    serializers: {
        req: (req) => {
            if (devMode) {
                return `${req.method} ${req.url} [${req.id}]`;
            } else {
                return {
                    id: req.id,
                    method: req.method,
                    url: req.url,
                    headers: req.headers,
                    remoteAddress: req.connection && req.connection.remoteAddress,
                    remotePort: req.connection && req.connection.remotePort
                };
            }
        },
        res: (res) => {
            if (devMode) {
                const cleanedHeaders = String(res.header)
                    .replace(/(\r\n)+/g, ' • ')
                    .replace(/\s•\s$/, '');
                return `[${res.statusCode}] ${cleanedHeaders}`;
            } else {
                return {
                    statusCode: res.statusCode,
                    header: res.header
                };
            }
        }
    }
});

module.exports = function generateServer() {
    const app = express();
    const router = express.Router(); // eslint-disable-line new-cap

    app.disable('x-powered-by');
    app.use(httpLogger);
    app.use(sentry.requestHandler());
    app.use(bodyParser.json());
    app.use(bodyParser.urlencoded({extended: true}));
    app.use(bearerToken());
    // FIXME
    // app.use((req, res, next) => {
    //     if (req.token) {
    //         return AccessToken.findByToken(req.token)
    //             .then(token => {
    //                 req.user = {
    //                     id: token.id,
    //                     type: 'robot',
    //                     name: token.name,
    //                     roles: token.roles
    //                 };
    //                 next();
    //             })
    //             .catch(error => {
    //                 next(error);
    //             });
    //     } else {
    //         req.user = {
    //             id: null,
    //             roles: {}
    //         };
    //         return next();
    //     }
    // });
    app.use(cookieParser());

    if (devMode) {
        app.use(serveStatic(`${__dirname}/../../public`));
    }

    initRoutes(app);
    app.use('/', router);

    app.use(sentry.errorHandler());

    app.use((err, req, res, next) => { // eslint-disable-line no-unused-vars
        if (req.log) {
            req.log.error(err, 'Unhandled error');
        } else {
            pino.error(err, 'Unhandled error');
        }
        res.status(500).send('Internal server error');
    });

    return function listen(port) {
        const server = http.createServer(app);
        server.on('listening', () => {
            pino.info(`Express server is listening on port ${(server.address()).port}`);
        });
        server.on('error', (error) => {
            if ((error).syscall !== 'listen') {
                throw error;
            }
            switch ((error).code) {
                case 'EACCES':
                    pino.error(`Port ${port} requires elevated privileges`);
                    process.exit(1);
                    break;
                case 'EADDRINUSE':
                    pino.error(`Port ${port} is already in use`);
                    process.exit(1);
                    break;
                default:
                    throw error;
            }
        });
        server.listen(port);
    };
};
